from __future__ import annotations

__all__ = ["Cpu"]

from typing import cast

from ._types import Content
from .base import BaseEndpoint


class Cpu(BaseEndpoint):
    """Cpu endpoint"""

    __path = "cpu"

    __slots__ = ()

    def get_info(self) -> Content:
        """Returns information about cpu usage."""

        result = self._client.get(f"{self.__path}/")
        return cast(Content, result.content)



from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.mark.usefixtures("paw_api_client")
class TestStudents:

    def test_list(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/students/", json={})
        res = self.api.students.list()
        assert isinstance(res, list), res

    @pytest.mark.parametrize("student", ("student1", "anotherstudent2"))
    def test_delete(self, mock_responses: 'RequestsMock', student: str) -> None:
        mock_responses.delete(f"{self.url}/students/{student}/", status=204)
        assert self.api.students.delete(student)



from enum import Enum


class Shell(Enum):
    """Available executable for console.

    See :meth:`pawapi.api.Console.create()`
    """

    BASH = "bash"
    MYSQL = "mysql"
    PYTHON310 = "python3.10"
    PYTHON39 = "python3.9"
    PYTHON38 = "python3.8"
    PYTHON37 = "python3.7"
    PYTHON36 = "python3.6"
    IPYTHON310 = "ipython3.10"
    IPYTHON39 = "ipython3.9"
    IPYTHON38 = "ipython3.8"
    IPYTHON37 = "ipython3.7"
    IPYTHON36 = "ipython3.6"
    PYPY2 = "pypy2"
    PYPY3 = "pypy3"



from enum import Enum


class SystemImage(Enum):
    """System image.

    .. note::

        Use :meth:`pawapi.api.System.get_current_image()` to get list of
        available system images.

    See: https://help.pythonanywhere.com/pages/ChangingSystemImage/
    """

    DANGERMOUSE = "dangermouse"
    EARLGREY = "earlgrey"
    FISHNCHIPS = "fishnchips"
    GLASTONBURY = "glastonbury"
    HAGGIS = "haggis"



from typing import TYPE_CHECKING

import pytest
from responses.matchers import multipart_matcher
from responses.matchers import query_param_matcher
from responses.matchers import urlencoded_params_matcher

if TYPE_CHECKING:
    from responses import RequestsMock

FILE_PATH = "/home/user123/file.md"
DIR_PATH = "/home/user123/dir"


@pytest.mark.usefixtures("paw_api_client")
class TestFiles:

    def test_get_file_content(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/files/path{FILE_PATH}",
            body=b"foo",
            content_type="application/octet-stream",
        )
        res = self.api.file.get_file_content(FILE_PATH)
        assert res is not None, res

    def test_get_file_content_not_a_file(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.get(
            f"{self.url}/files/path{FILE_PATH}",
            json={"foo": {}},
        )
        res = self.api.file.get_file_content(FILE_PATH)
        assert res is None, res

    def test_get_directory_content(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.get(
            f"{self.url}/files/path{DIR_PATH}",
            json={
                "foo": {
                    "type": "file",
                    "url": "/foo",
                },
                "bar": {
                    "type": "directory",
                    "url": "/bar",
                },
            },
        )
        res = self.api.file.get_directory_content(DIR_PATH)
        assert res is not None, res

    def test_get_directory_content_not_a_dir(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.get(
            f"{self.url}/files/path{DIR_PATH}",
            body=b"bar",
            content_type="application/octet-stream",
        )
        res = self.api.file.get_directory_content(DIR_PATH)
        assert res is None, res

    def test_upload_file_new(self, mock_responses: 'RequestsMock') -> None:
        content = b"file"
        mock_responses.post(
            url=f"{self.url}/files/path{FILE_PATH}",
            match=(multipart_matcher({"content": content}), ),
            status=201,
        )
        assert self.api.file.upload(path=FILE_PATH, file_content=content)

    def test_upload_file_update(self, mock_responses: 'RequestsMock') -> None:
        content = b"file"
        mock_responses.post(
            url=f"{self.url}/files/path{FILE_PATH}",
            match=(multipart_matcher({"content": content}), ),
            status=200,
        )
        assert not self.api.file.upload(path=FILE_PATH, file_content=content)

    def test_delete_file(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(f"{self.url}/files/path{FILE_PATH}", status=204)
        assert self.api.file.delete(FILE_PATH)

    def test_start_sharing_file(self, mock_responses: 'RequestsMock') -> None:
        path = "/foo/bar/"
        mock_responses.post(
            url=f"{self.url}/files/sharing/",
            match=(urlencoded_params_matcher({"path": FILE_PATH}), ),
            json={"url": path},
        )
        res = self.api.file.start_sharing(path=FILE_PATH)
        assert res == path

    def test_get_sharing_status(self, mock_responses: 'RequestsMock') -> None:
        path = "/foo/bar/"
        mock_responses.get(
            url=f"{self.url}/files/sharing/",
            match=(query_param_matcher({"path": FILE_PATH}), ),
            json={"url": path},
        )
        res = self.api.file.get_sharing_status(path=FILE_PATH)
        assert res == path

    def test_get_sharing_status_none(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.get(
            url=f"{self.url}/files/sharing/",
            match=(query_param_matcher({"path": FILE_PATH}), ),
            status=404
        )
        res = self.api.file.get_sharing_status(path=FILE_PATH)
        assert res is None

    def test_stop_sharing_file(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(
            url=f"{self.url}/files/sharing/",
            match=(query_param_matcher({"path": FILE_PATH}), ),
            status=204,
        )
        assert self.api.file.stop_sharing(path=FILE_PATH)

    def test_get_tree(self, mock_responses: 'RequestsMock') -> None:
        path = "/home/user/"
        mock_responses.get(
            url=f"{self.url}/files/tree/",
            match=(query_param_matcher({"path": path}), ),
            content_type="application/json",
            body=b"[]",
        )
        res = self.api.file.get_tree(path=path)
        assert isinstance(res, list)

    def test_get_tree_not_found(self, mock_responses: 'RequestsMock') -> None:
        path = "/home/user/"
        mock_responses.get(
            url=f"{self.url}/files/tree/",
            match=(query_param_matcher({"path": path}), ),
            status=400,
        )
        res = self.api.file.get_tree(path=path)
        assert res is None



from __future__ import annotations

__all__ = ["Client"]

from dataclasses import dataclass
from typing import Dict
from typing import Optional
from typing import Union

import requests

from .exceptions import BadRequestError
from .exceptions import InvalidJSONError
from .exceptions import InvalidTokenError
from .exceptions import NotFoundError
from .exceptions import PermissionDeniedError
from .exceptions import RequestTimeoutError
from .exceptions import StatusError
from .region import Region

Content = Union[Dict[str, Union[str, int, bytes]], bytes]


class Client:
    """Http client.

    :param username: PythonAnywhere username.
    :param token: API token.
    :param region: Account region.
    :param timeout: Request timeout.
    """

    __slots__ = (
        "_url",
        "_headers",
        "_timeout",
    )

    def __init__(
        self,
        username: str,
        token: str,
        region: Region,
        timeout: float,
    ) -> None:
        self._url = "https://{}.pythonanywhere.com/api/v0/user/{}".format(
            "eu" if region is Region.EU else "www", username
        )
        self._headers = {"Authorization": f"Token {token}"}
        self._timeout = timeout

    def _request(
        self,
        method: str,
        path: str,
        *,
        data: Optional[Dict[str, Optional[Union[str, int]]]] = None,
        params: Optional[Dict[str, Optional[Union[str, int]]]] = None,
        files: Optional[Dict[str, bytes]] = None,
    ) -> Response:
        try:
            response = requests.request(
                method=method,
                url=f"{self._url}/{path}",
                params=params,
                data=data,
                files=files,
                headers=self._headers,
                timeout=self._timeout,
            )
        except requests.Timeout:
            raise RequestTimeoutError(
                f"Request timed out (timeout={self._timeout})"
            ) from None

        if response.status_code >= 300:
            self._raise_for_status(response)

        return self._parse_content(response)

    def _raise_for_status(self, response: requests.Response) -> None:
        try:
            json = response.json()
        except requests.JSONDecodeError:
            json = {}

        status_code = response.status_code
        details = {
            "status_code": status_code,
            "description": json.get("error") or json.get("detail"),
            "raw_content": response.content,
        }

        if status_code == 400:
            raise BadRequestError(**details)
        elif status_code == 401:
            raise InvalidTokenError(**details)
        elif status_code == 401:
            raise InvalidTokenError(**details)
        elif status_code == 403:
            raise PermissionDeniedError(**details)
        elif status_code == 404:
            raise NotFoundError(**details)
        else:
            raise StatusError(**details)

    def _parse_content(self, response: requests.Response) -> Response:
        content: Optional[Content] = None
        content_type = response.headers.get("Content-Type", None)
        if response.content and content_type is not None:
            # response from /webapps with "text/json" in Content-Type
            if "/json" in content_type:  # json
                try:
                    content = response.json()
                except requests.JSONDecodeError:
                    raise InvalidJSONError(
                        f"Got invalid json: {response.content!r}"
                    ) from None
            elif "application/octet-stream" in content_type:  # file
                content = response.content
        return Response(status=response.status_code, content=content)

    def get(
        self,
        path: str,
        params: Optional[Dict[str, Optional[Union[str, int]]]] = None,
    ) -> Response:
        return self._request("get", path, params=params)

    def post(
        self,
        path: str,
        *,
        data: Optional[Dict[str, Optional[Union[str, int]]]] = None,
        files: Optional[Dict[str, bytes]] = None,
    ) -> Response:
        return self._request("post", path, data=data, files=files)

    def patch(
        self,
        path: str,
        data: Optional[Dict[str, Optional[Union[str, int]]]] = None,
    ) -> Response:
        return self._request("patch", path, data=data)

    def delete(
        self,
        path: str,
        params: Optional[Dict[str, Optional[Union[str, int]]]] = None,
    ) -> Response:
        return self._request("delete", path, params=params)


@dataclass(frozen=True)
class Response:
    status: int
    content: Optional[Content] = None



from typing import TYPE_CHECKING

import pytest
from responses.matchers import urlencoded_params_matcher

from pawapi.system import SystemImage

if TYPE_CHECKING:
    from responses import RequestsMock


def test_system_image() -> None:
    assert SystemImage.DANGERMOUSE.value == "dangermouse"
    assert SystemImage.EARLGREY.value == "earlgrey"
    assert SystemImage.FISHNCHIPS.value == "fishnchips"
    assert SystemImage.GLASTONBURY.value == "glastonbury"
    assert SystemImage.HAGGIS.value == "haggis"


@pytest.mark.usefixtures("paw_api_client")
class TestSystem:

    def test_get_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/system_image/", json={})
        res = self.api.system.get_current_image()
        assert isinstance(res, dict), res

    def test_set_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.patch(
            url=f"{self.url}/system_image/",
            match=(
                urlencoded_params_matcher({"system_image": "glastonbury"}),
            ),
            json={},
        )
        res = self.api.system.set_image(SystemImage.GLASTONBURY)
        assert isinstance(res, dict), res



from typing import TYPE_CHECKING

import pytest
from responses.matchers import urlencoded_params_matcher

from pawapi.python import Python2
from pawapi.python import Python3

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.mark.usefixtures("paw_api_client")
class TestPython:

    def test_get_python_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/default_python_version/", json={})
        res = self.api.python.get_python_version()
        assert isinstance(res, dict)

    def test_set_python_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.patch(
            url=f"{self.url}/default_python_version/",
            match=(
                urlencoded_params_matcher({"default_python_version": "2.7"}),
            ),
            json={},
        )
        res = self.api.python.set_python_version(Python2.PYTHON27)
        assert isinstance(res, dict), res

    def test_get_python3_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/default_python3_version/", json={})
        res = self.api.python.get_python3_version()
        assert isinstance(res, dict), res

    def test_set_python3_version(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.patch(
            url=f"{self.url}/default_python3_version/",
            match=(
                urlencoded_params_matcher({"default_python3_version": "3.8"}),
            ),
            json={},
        )
        res = self.api.python.set_python3_version(Python3.PYTHON38)
        assert isinstance(res, dict), res

    def test_get_python_sar_version(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.get(
            f"{self.url}/default_save_and_run_python_version/",
            json={},
        )
        res = self.api.python.get_sar_version()
        assert isinstance(res, dict), res

    def test_set_python_sar_version(
        self, mock_responses: 'RequestsMock'
    ) -> None:
        mock_responses.patch(
            url=f"{self.url}/default_save_and_run_python_version/",
            match=(
                urlencoded_params_matcher({
                    "default_save_and_run_python_version": "3.7",
                }),
            ),
            json={},
        )
        res = self.api.python.set_sar_version(Python3.PYTHON37)
        assert isinstance(res, dict), res



from typing import TYPE_CHECKING
from typing import Any
from typing import Dict

import pytest
from responses.matchers import urlencoded_params_matcher

from pawapi.interval import TaskInterval

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.fixture
def params_create_hourly() -> Dict[str, Any]:
    return {
        "command": "echo 123 >> file",
        "enabled": True,
        "interval": "hourly",
        "minute": 12,
        "description": "Schedule hourly description",
    }


@pytest.fixture
def params_create_daily(params_create_hourly: Dict[str, Any]) -> Dict[str, Any]:
    params_create_hourly.update({
        "hour": 12,
        "interval": "daily",
        "description": "Schedule daily description",
    })
    return params_create_hourly


def test_schedule_interval() -> None:
    assert TaskInterval.DAILY.value == "daily"
    assert TaskInterval.HOURLY.value == "hourly"


TASK_ID = 12


@pytest.mark.usefixtures("paw_api_client")
class TestScheduledTask:

    def test_list(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/schedule/",
            body=b"[]",
            content_type="application/json",
        )
        res = self.api.scheduled_task.list()
        assert isinstance(res, list), res

    def test_create_hourly(
        self,
        mock_responses: 'RequestsMock',
        params_create_hourly: Dict[str, Any],
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/schedule/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_create_hourly.items()
                }),
            ),
            json={},
        )
        params_create_hourly["interval"] = TaskInterval.HOURLY
        res = self.api.scheduled_task.create(**params_create_hourly)
        assert isinstance(res, dict), res

    def test_create_daily(
        self,
        mock_responses: 'RequestsMock',
        params_create_daily: Dict[str, Any],
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/schedule/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_create_daily.items()
                }),
            ),
            json={},
        )
        params_create_daily["interval"] = TaskInterval.DAILY
        res = self.api.scheduled_task.create(**params_create_daily)
        assert isinstance(res, dict), res

    def test_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/schedule/{TASK_ID}/", json={})
        res = self.api.scheduled_task.get_info(TASK_ID)
        assert isinstance(res, dict)

    def test_update_full(
        self,
        mock_responses: 'RequestsMock',
        params_create_daily: Dict[str, Any],
    ) -> None:
        params_create_daily.update({
            "hour": 21,
            "minute": 21,
            "description": "Updated description",
        })
        mock_responses.patch(
            url=f"{self.url}/schedule/{TASK_ID}/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_create_daily.items()
                }),
            ),
            json={},
        )
        params_create_daily["interval"] = TaskInterval(
            params_create_daily["interval"]
        )
        res = self.api.scheduled_task.update(
            task_id=TASK_ID,
            **params_create_daily,
        )
        assert isinstance(res, dict), res

    def test_update_time(self, mock_responses: 'RequestsMock') -> None:
        params = {"hour": 12, "minute": 12}
        mock_responses.patch(
            url=f"{self.url}/schedule/{TASK_ID}/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params.items()
                }),
            ),
            json={},
        )
        res = self.api.scheduled_task.update(task_id=TASK_ID, **params)
        assert isinstance(res, dict), res

    def test_delete(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(f"{self.url}/schedule/{TASK_ID}/", status=204)
        assert self.api.scheduled_task.delete(TASK_ID)



import pytest
from responses import RequestsMock

from pawapi import Pawapi


@pytest.fixture(scope="session")
def test_user() -> str:
    return "testUser"


@pytest.fixture(scope="session")
def test_token() -> str:
    return "testToken"


@pytest.fixture
def pawapi(test_user: str, test_token: str) -> Pawapi:
    return Pawapi(test_user, test_token)


@pytest.fixture
def paw_api_client(pawapi: Pawapi, request: pytest.FixtureRequest) -> None:
    request.cls.api = pawapi
    request.cls.url = pawapi._client._url


@pytest.fixture
def mock_responses() -> RequestsMock:
    with RequestsMock() as r:
        yield r



from typing import TYPE_CHECKING
from typing import Dict
from typing import Union

import pytest
from responses.matchers import urlencoded_params_matcher

from pawapi.shell import Shell

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.fixture
def params_create() -> Dict[str, str]:
    return {
        "executable": "python3.9",
        "working_directory": "/home",
    }


@pytest.fixture(params=(10, "10"))
def console_id(request: pytest.FixtureRequest) -> Union[int, str]:
    return request.param


@pytest.mark.usefixtures("paw_api_client")
class TestConsole:

    def test_list(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/consoles/",
            body=b"[]",
            content_type="appclication/json",
        )
        res = self.api.console.list()
        assert isinstance(res, list), res

    def test_list_shared(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/consoles/shared_with_you/",
            body=b"[]",
            content_type="appclication/json",
        )
        res = self.api.console.list_shared()
        assert isinstance(res, list), res

    def test_create_python3(
        self, mock_responses: 'RequestsMock', params_create: Dict[str, str]
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/consoles/",
            match=(urlencoded_params_matcher({**params_create}), ),
            json={},
            status=201,
        )
        params_create["executable"] = Shell(params_create["executable"])
        res = self.api.console.create(**params_create)
        assert isinstance(res, dict), res

    def test_create_fail(
        self, mock_responses: 'RequestsMock', params_create: Dict[str, str]
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/consoles/",
            match=(urlencoded_params_matcher({**params_create}), ),
        )
        params_create["executable"] = Shell(params_create["executable"])
        res = self.api.console.create(**params_create)
        assert res is None

    def test_create_bash(
        self, mock_responses: 'RequestsMock', params_create: Dict[str, str]
    ) -> None:
        params_create["executable"] = "bash"
        mock_responses.post(
            url=f"{self.url}/consoles/",
            match=(urlencoded_params_matcher({**params_create}), ),
            json={},
            status=201,
        )
        params_create["executable"] = Shell(params_create["executable"])
        res = self.api.console.create(**params_create)
        assert isinstance(res, dict), res

    def test_create_with_args(
        self, mock_responses: 'RequestsMock', params_create: Dict[str, str]
    ) -> None:
        params_create["arguments"] = "--useful-arg"
        mock_responses.post(
            url=f"{self.url}/consoles/",
            match=(urlencoded_params_matcher({**params_create}), ),
            json={},
            status=201,
        )
        params_create["executable"] = Shell(params_create["executable"])
        res = self.api.console.create(**params_create)
        assert isinstance(res, dict), res

    def test_info(
        self, mock_responses: 'RequestsMock', console_id: int
    ) -> None:
        mock_responses.get(f"{self.url}/consoles/{console_id}/", json={})
        res = self.api.console.get_info(console_id)
        assert isinstance(res, dict), res

    def test_kill(
        self, mock_responses: 'RequestsMock', console_id: int
    ) -> None:
        mock_responses.delete(f"{self.url}/consoles/{console_id}/", status=204)
        assert self.api.console.kill(console_id)

    def test_get_output(
        self, mock_responses: 'RequestsMock', console_id: int
    ) -> None:
        output = "hello world!\n"
        mock_responses.get(
            f"{self.url}/consoles/{console_id}/get_latest_output/",
            json={"output": output}
        )
        assert self.api.console.get_output(console_id) == output

    def test_send_input(
        self, mock_responses: 'RequestsMock', console_id: int
    ) -> None:
        console_input = "echo 123"
        mock_responses.post(
            url=f"{self.url}/consoles/{console_id}/send_input/",
            match=(urlencoded_params_matcher({"input": console_input}), ),
            json={"status": "OK"},
        )
        assert self.api.console.send_input(console_id, command=console_input)



from typing import Union

import pytest

from pawapi.python import Python2
from pawapi.python import Python3
from pawapi.shell import Shell


def test_console_executable() -> None:
    assert Shell.BASH.value == "bash"
    assert Shell.MYSQL.value == "mysql"
    assert Shell.PYTHON310.value == "python3.10"
    assert Shell.PYTHON39.value == "python3.9"
    assert Shell.PYTHON38.value == "python3.8"
    assert Shell.PYTHON37.value == "python3.7"
    assert Shell.PYTHON36.value == "python3.6"
    assert Shell.IPYTHON310.value == "ipython3.10"
    assert Shell.IPYTHON39.value == "ipython3.9"
    assert Shell.IPYTHON38.value == "ipython3.8"
    assert Shell.IPYTHON37.value == "ipython3.7"
    assert Shell.IPYTHON36.value == "ipython3.6"
    assert Shell.PYPY2.value == "pypy2"
    assert Shell.PYPY3.value == "pypy3"


def test_python2_version() -> None:
    assert Python2.PYTHON27.value == "python2.7"


def test_python3_version() -> None:
    assert Python3.PYTHON310.value == "python3.10"
    assert Python3.PYTHON39.value == "python3.9"
    assert Python3.PYTHON38.value == "python3.8"
    assert Python3.PYTHON37.value == "python3.7"
    assert Python3.PYTHON36.value == "python3.6"


@pytest.mark.parametrize(
    "version,result",
    (
        (Python3.PYTHON310, "3.10"),
        (Python3.PYTHON39, "3.9"),
        (Python3.PYTHON37, "3.7"),
        (Python2.PYTHON27, "2.7"),
    )
)
def test_format_version_short(
    version: Union[Python2, Python3], result: str
) -> None:
    assert version.format_value(short=True) == result


@pytest.mark.parametrize(
    "version,result",
    (
        (Python3.PYTHON310, "python310"),
        (Python3.PYTHON39, "python39"),
        (Python3.PYTHON37, "python37"),
        (Python2.PYTHON27, "python27"),
    )
)
def test_format_version_no_dot(
    version: Union[Python2, Python3], result: str
) -> None:
    assert version.format_value(dot=False) == result


@pytest.mark.parametrize(
    "version,result",
    (
        (Python3.PYTHON310, "310"),
        (Python3.PYTHON39, "39"),
        (Python3.PYTHON37, "37"),
        (Python2.PYTHON27, "27"),
    )
)
def test_format_version_short_no_dot(
    version: Union[Python2, Python3], result: str
) -> None:
    assert version.format_value(short=True, dot=False) == result



from __future__ import annotations

__all__ = [
    "Python2",
    "Python3",
]

from enum import Enum


class _PythonEnum(Enum):

    def format_value(self, *, short: bool = False, dot: bool = True) -> str:
        ret: str = self.value
        if short:
            ret = ret[6:]
        if not dot:
            ret = ret.replace(".", "")
        return ret


class Python2(_PythonEnum):
    """Available Python 2 versions"""

    PYTHON27 = "python2.7"


class Python3(_PythonEnum):
    """Available Python 3 version"""

    PYTHON310 = "python3.10"
    PYTHON39 = "python3.9"
    PYTHON38 = "python3.8"
    PYTHON37 = "python3.7"
    PYTHON36 = "python3.6"



from __future__ import annotations

__all__ = ["Pawapi"]

from typing import Optional

from .api import AlwaysOnTask
from .api import Console
from .api import Cpu
from .api import File
from .api import Python
from .api import ScheduledTask
from .api import Students
from .api import System
from .api import Webapp
from .client import Client
from .region import Region
from .utils import validate_credentials


class Pawapi:
    """Main `pawapi` object.

    Usage::

        token = '<your_token>'
        user = '<your_username>'

        api = Pawapi(user, token)
        print(api.cpu.get_info())

    :param username: PythonAnywhere username.
    :param token: API token.
    :param region: Account region.
    :param timeout: Request timeout.
    """

    __slots__ = (
        "_username",
        "_region",
        "_client",
        "_alwayson_task",
        "_console",
        "_cpu",
        "_file",
        "_python",
        "_scheduled_task",
        "_students",
        "_system",
        "_webapp",
    )

    def __init__(
        self,
        username: str,
        token: str,
        *,
        region: Region = Region.US,
        timeout: float = 8,
    ) -> None:
        validate_credentials("username", username)
        validate_credentials("token", token)

        self._username = username
        self._region = region

        self._client = Client(
            username=username,
            token=token,
            region=region,
            timeout=timeout,
        )

        self._alwayson_task: Optional[AlwaysOnTask] = None
        self._console: Optional[Console] = None
        self._cpu: Optional[Cpu] = None
        self._file: Optional[File] = None
        self._python: Optional[Python] = None
        self._scheduled_task: Optional[ScheduledTask] = None
        self._students: Optional[Students] = None
        self._system: Optional[System] = None
        self._webapp: Optional[Webapp] = None

    def __str__(self) -> str:
        return f"{self._region.value}:{self._username}"

    def __repr__(self) -> str:
        return f"<Pawapi [{self._region.value}:{self._username}]>"

    @property
    def username(self) -> str:
        return self._username

    @property
    def region(self) -> Region:
        return self._region

    @property
    def console(self) -> Console:
        """Console endpoint.

        See: :class:`pawapi.api.Console`
        """

        if self._console is None:
            self._console = Console(self._client)
        return self._console

    @property
    def cpu(self) -> Cpu:
        """Cpu endpoint.

        See: :class:`pawapi.api.Cpu`
        """

        if self._cpu is None:
            self._cpu = Cpu(self._client)
        return self._cpu

    @property
    def file(self) -> File:
        """File endpoint.

        See: :class:`pawapi.api.File`
        """

        if self._file is None:
            self._file = File(self._client)
        return self._file

    @property
    def python(self) -> Python:
        """Python endpoint.

        See: :class:`pawapi.api.Python`
        """

        if self._python is None:
            self._python = Python(self._client)
        return self._python

    @property
    def scheduled_task(self) -> ScheduledTask:
        """Scheduled task ednpoint.

        See: :class:`pawapi.api.ScheduledTask`
        """

        if self._scheduled_task is None:
            self._scheduled_task = ScheduledTask(self._client)
        return self._scheduled_task

    @property
    def students(self) -> Students:
        """Students endpoint.

        See: :class:`pawapi.api.Students`
        """

        if self._students is None:
            self._students = Students(self._client)
        return self._students

    @property
    def system(self) -> System:
        """System endpoint.

        See: :class:`pawapi.api.System`
        """

        if self._system is None:
            self._system = System(self._client)
        return self._system

    @property
    def alwayson_task(self) -> AlwaysOnTask:
        """Always-On task endpoint

        See: :class:`pawapi.api.AlwaysOnTask`
        """

        if self._alwayson_task is None:
            self._alwayson_task = AlwaysOnTask(self._client)
        return self._alwayson_task

    @property
    def webapp(self) -> Webapp:
        """Webapp endpoint.

        See: :class:`pawapi.api.Webapp`
        """

        if self._webapp is None:
            self._webapp = Webapp(self._client)
        return self._webapp



from __future__ import annotations

__all__ = ["Students"]

from typing import Dict
from typing import List
from typing import cast

from ._types import Content
from .base import BaseEndpoint


class Students(BaseEndpoint):
    """Students endpoint"""

    __path = "students"

    __slots__ = ()

    def list(self) -> List[Content]:
        """List of students."""

        result = self._client.get(f"{self.__path}/")
        content = cast(Dict[str, List[Content]], result.content)
        return content.get("students", [])

    def delete(self, student: str) -> bool:
        """Delete a student.

        :param student: Student id.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{student}/")
        return result.status == 204



from typing import TYPE_CHECKING
from typing import Dict
from typing import Union

import pytest
from responses.matchers import urlencoded_params_matcher

from pawapi.exceptions import BadRequestError
from pawapi.python import Python3

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.fixture
def params_static_files() -> Dict[str, str]:
    return {
        "url": "/static",
        "path": "/home/user/static",
    }


@pytest.fixture
def params_static_headers() -> Dict[str, str]:
    return {
        "url": "/header",
        "name": "headername",
        "value": "headervalue",
    }


@pytest.fixture(params=(55, "55"))
def file_id(request: pytest.FixtureRequest) -> Union[int, str]:
    return request.param


@pytest.fixture(params=(44, "44"))
def header_id(request: pytest.FixtureRequest) -> Union[int, str]:
    return request.param


@pytest.mark.usefixtures("paw_api_client")
class TestWebApp:
    domain = "test.domain.site"
    file_id = 321
    header_id = 123

    def test_list(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/webapps/")
        self.api.webapp.list()

    def test_create(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.post(
            url=f"{self.url}/webapps/",
            match=(
                urlencoded_params_matcher({
                    "domain_name": self.domain,
                    "python_version": "python38",
                }),
            ),
        )
        assert self.api.webapp.create(
            domain_name=self.domain,
            python_version=Python3.PYTHON38,
        )

    def test_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/webapps/{self.domain}/", json={})
        res = self.api.webapp.get_info(self.domain)
        assert isinstance(res, dict), res

    def test_update(self, mock_responses: 'RequestsMock') -> None:
        params = {
            "source_directory": "/home/user/appsrc",
            "virtualenv_path": "/home/user/venv",
            "force_https": True,
        }
        mock_responses.patch(
            url=f"{self.url}/webapps/{self.domain}/",
            match=(
                urlencoded_params_matcher({
                    "python_version": "3.9",
                    **{k: str(v)
                       for k, v in params.items()}
                }),
            ),
            json={},
        )
        res = self.api.webapp.update(
            domain_name=self.domain,
            python_version=Python3.PYTHON39,
            **params,
        )
        assert isinstance(res, dict), res

    def test_update_protected(self, mock_responses: 'RequestsMock') -> None:
        user = "username"
        pwd = "password123"

        mock_responses.patch(
            url=f"{self.url}/webapps/{self.domain}/",
            match=(
                urlencoded_params_matcher({
                    "password_protection_enabled": "True",
                    "password_protection_username": user,
                    "password_protection_password": pwd,
                }),
            ),
            json={},
        )
        res = self.api.webapp.update(
            domain_name=self.domain,
            protection=True,
            protection_username=user,
            protection_password=pwd,
        )
        assert isinstance(res, dict), res

    def test_delete(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(f"{self.url}/webapps/{self.domain}/", status=204)
        assert self.api.webapp.delete(self.domain)

    def test_enable(self, mock_responses: 'RequestsMock') -> None:
        json = {"status": "OK"}
        mock_responses.post(
            f"{self.url}/webapps/{self.domain}/enable/", json=json
        )
        assert self.api.webapp.enable(self.domain)

    def test_enable_enabled(self, mock_responses: 'RequestsMock') -> None:
        json = {"error": "You cannot create any more webapps - ..."}
        mock_responses.post(
            f"{self.url}/webapps/{self.domain}/enable/",
            json=json,
            status=400,
        )
        assert self.api.webapp.enable(self.domain)

    def test_disable(self, mock_responses: 'RequestsMock') -> None:
        json = {"status": "OK"}
        mock_responses.post(
            f"{self.url}/webapps/{self.domain}/disable/", json=json
        )
        assert self.api.webapp.disable(self.domain)

    def test_reload(self, mock_responses: 'RequestsMock') -> None:
        json = {"status": "OK"}
        mock_responses.post(
            f"{self.url}/webapps/{self.domain}/reload/", json=json
        )
        assert self.api.webapp.reload(self.domain)

    def test_reload_disabled(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.post(
            f"{self.url}/webapps/{self.domain}/reload/",
            status=500,
        )
        assert not self.api.webapp.reload(self.domain)

    def test_ssl_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/webapps/{self.domain}/ssl/", json={})
        res = self.api.webapp.get_ssl_info(self.domain)
        assert isinstance(res, dict), res

    def test_add_ssl(self, mock_responses: 'RequestsMock') -> None:
        params = {
            "cert": "cert",
            "private_key": "privete_key",
        }
        mock_responses.post(
            url=f"{self.url}/webapps/{self.domain}/ssl/",
            match=(urlencoded_params_matcher(params), ),
            json={"status": "OK"},
        )
        assert self.api.webapp.add_ssl(domain_name=self.domain, **params)

    def test_add_ssl_invalid_cred(self, mock_responses: 'RequestsMock') -> None:
        params = {
            "cert": "cert",
            "private_key": "privete_key",
        }
        mock_responses.post(
            url=f"{self.url}/webapps/{self.domain}/ssl/",
            match=(urlencoded_params_matcher(params), ),
            status=400,
        )
        with pytest.raises(BadRequestError):
            self.api.webapp.add_ssl(domain_name=self.domain, **params)

    def test_delete_ssl(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(
            f"{self.url}/webapps/{self.domain}/ssl/",
            json={"status": "OK"},
        )
        assert self.api.webapp.delete_ssl(self.domain)

    def test_list_static_files(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/webapps/{self.domain}/static_files/",
            body=b"[]",
            content_type="application/json",
        )
        res = self.api.webapp.list_static_files(self.domain)
        assert isinstance(res, list)

    def test_add_static_file(
        self,
        mock_responses: 'RequestsMock',
        params_static_files: Dict[str, str],
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/webapps/{self.domain}/static_files/",
            match=(urlencoded_params_matcher(params_static_files), ),
            json={},
        )
        res = self.api.webapp.add_static_file(
            domain_name=self.domain,
            **params_static_files,
        )
        assert isinstance(res, dict)

    def test_static_file_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/webapps/{self.domain}/static_files/{self.file_id}/",
            json={},
        )
        res = self.api.webapp.get_static_file_info(self.domain, self.file_id)
        assert isinstance(res, dict)

    def test_update_static_file(
        self,
        mock_responses: 'RequestsMock',
        params_static_files: Dict[str, str],
    ) -> None:
        mock_responses.patch(
            url=f"{self.url}/webapps/{self.domain}/static_files/{self.file_id}/",  # noqa: E501
            match=(urlencoded_params_matcher(params_static_files), ),
            json={},
        )  # yapf: disable
        res = self.api.webapp.update_static_file(
            domain_name=self.domain,
            file_id=self.file_id,
            **params_static_files,
        )
        assert isinstance(res, dict)

    def test_delete_static_file(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(
            f"{self.url}/webapps/{self.domain}/static_files/{self.file_id}/",
            status=204,
        )
        assert self.api.webapp.delete_static_file(self.domain, self.file_id)

    def test_list_static_headers(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/webapps/{self.domain}/static_headers/",
            content_type="application/json",
            body=b"[]",
        )
        res = self.api.webapp.list_static_headers(self.domain)
        assert isinstance(res, list)

    def test_add_static_header(
        self,
        mock_responses: 'RequestsMock',
        params_static_headers: Dict[str, str]
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/webapps/{self.domain}/static_headers/",
            match=(urlencoded_params_matcher(params_static_headers), ),
            json={},
        )
        res = self.api.webapp.add_static_header(
            domain_name=self.domain,
            **params_static_headers,
        )
        assert isinstance(res, dict)

    def test_static_header_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(
            f"{self.url}/webapps/{self.domain}/static_headers/{self.header_id}/",  # noqa: E501
            json={},
        )
        res = self.api.webapp.get_static_header_info(
            self.domain,
            self.header_id,
        )
        assert isinstance(res, dict)

    def test_update_static_header(
        self,
        mock_responses: 'RequestsMock',
        params_static_headers: Dict[str, str],
    ) -> None:
        mock_responses.patch(
            url=f"{self.url}/webapps/{self.domain}/static_headers/{self.header_id}/",  # noqa: E501
            match=(urlencoded_params_matcher(params_static_headers), ),
            json={},
        )  # yapf: disable
        res = self.api.webapp.update_static_header(
            domain_name=self.domain,
            header_id=self.header_id,
            **params_static_headers,
        )
        assert isinstance(res, dict)

    def test_delete_static_header(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(
            f"{self.url}/webapps/{self.domain}/static_headers/{self.header_id}/",  # noqa: E501
            status=204,
        )
        assert self.api.webapp.delete_static_header(self.domain, self.header_id)



from typing import TYPE_CHECKING
from typing import Any
from typing import Dict

import pytest
from responses.matchers import urlencoded_params_matcher

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.fixture
def params_create() -> Dict[str, Any]:
    return {
        "command": "bash script.sh",
        "description": "Magic bash script",
        "enabled": True,
    }


@pytest.fixture
def params_update(params_create: Dict[str, Any]) -> Dict[str, Any]:
    params_create.update({
        "command": "python script.py",
        "description": "Magic python script",
    })
    return params_create


@pytest.mark.usefixtures("paw_api_client")
class TestAlwaysOnTask:
    task_id = 12

    def test_list(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/always_on/")
        self.api.alwayson_task.list()

    def test_create(
        self, mock_responses: 'RequestsMock', params_create: Dict[str, Any]
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/always_on/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_create.items()
                }),
            ),
            json={},
        )
        res = self.api.alwayson_task.create(**params_create)
        assert isinstance(res, dict), res

    def test_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/always_on/{self.task_id}/", json={})
        res = self.api.alwayson_task.get_info(self.task_id)
        assert isinstance(res, dict), res

    def test_update(
        self,
        mock_responses: 'RequestsMock',
        params_update: Dict[str, Any],
    ) -> None:
        mock_responses.patch(
            url=f"{self.url}/always_on/{self.task_id}/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_update.items()
                }),
            ),
            json={},
        )
        res = self.api.alwayson_task.update(
            task_id=self.task_id,
            **params_update,
        )
        assert isinstance(res, dict), res

    def test_delete(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.delete(
            f"{self.url}/always_on/{self.task_id}/", status=204
        )
        assert self.api.alwayson_task.delete(self.task_id)

    def test_restart(
        self,
        mock_responses: 'RequestsMock',
        params_update: Dict[str, Any],
    ) -> None:
        mock_responses.post(
            url=f"{self.url}/always_on/{self.task_id}/restart/",
            match=(
                urlencoded_params_matcher({
                    k: str(v)
                    for k, v in params_update.items()
                }),
            ),
            json={},
        )
        res = self.api.alwayson_task.restart(
            task_id=self.task_id,
            **params_update,
        )
        assert isinstance(res, dict)



import json
from typing import TYPE_CHECKING
from typing import Union

import pytest
from requests.exceptions import Timeout
from responses.matchers import header_matcher

from pawapi.client import Client
from pawapi.exceptions import InvalidCredentialsError
from pawapi.exceptions import InvalidTokenError
from pawapi.exceptions import NotFoundError
from pawapi.exceptions import PermissionDeniedError
from pawapi.exceptions import RequestTimeoutError
from pawapi.exceptions import StatusError
from pawapi.region import Region
from pawapi.utils import validate_credentials

if TYPE_CHECKING:
    from respsonses import RequestsMock


@pytest.fixture(scope="class")
def client(test_user: str, test_token: str) -> Client:
    return Client(test_user, test_token, Region.US, 8)


def test_region() -> None:
    assert Region.US.value == "us"
    assert Region.EU.value == "eu"


@pytest.mark.parametrize(
    "value",
    (
        "user_name",
        "username!",
        "",
        123,
        " ",
        "u   ",
        b"username",
        " username",
        None,
        "@username",
        "usern#21ame",
    ),
)
def test_validate_invalid_credentials(value):
    with pytest.raises(InvalidCredentialsError):
        validate_credentials("invalid", value)


@pytest.mark.parametrize(
    "value",
    (
        "username",
        "username1",
        "user1name",
        "1234567890",
    ),
)
def test_validate_valid_credentials(value):
    validate_credentials("valid", value)


class TestBaseClient:

    @pytest.mark.parametrize(
        "region,url",
        (
            (
                Region.US,
                "https://www.pythonanywhere.com/api/v0/user/",
            ),
            (
                Region.EU,
                "https://eu.pythonanywhere.com/api/v0/user/",
            ),
        )
    )
    def test_api_url(
        self,
        test_user: str,
        test_token: str,
        region: Region,
        url: str,
    ) -> None:
        client = Client(
            test_user,
            test_token,
            region=region,
            timeout=8,
        )
        assert client._url == url + test_user

    def test_api_auth_header(self, client: Client, test_token: str) -> None:
        assert client._headers.get(
            "Authorization", None
        ) == f"Token {test_token}"

    @pytest.mark.parametrize("content_type", ("application/json", "text/json"))
    def test_response_json(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
        test_token: str,
        content_type: str,
    ) -> None:
        json_params = {
            "list": [1, 2, 3, 4],
            "dict": {
                "key1": "value1",
                "key2": "value2",
            },
            "int": 12,
            "str": "str",
        }

        mock_responses.get(
            url=f"{client._url}/json/",
            content_type=content_type,
            body=json.dumps(json_params),
            match=(header_matcher({"Authorization": f"Token {test_token}"}), ),
        )
        resp = client.get("json/")
        assert resp.content == json_params

    @pytest.mark.parametrize(
        "content_type,body,result",
        (
            ("application/octet-stream", b"content", b"content"),
            ("application/json", b"", None),
            ("text/html", b"<h>html here</h>", None),
        )
    )
    def test_response_content(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
        content_type: str,
        test_token: str,
        body: bytes,
        result: Union[bytes, None],
    ) -> None:
        mock_responses.get(
            url=f"{client._url}/content/",
            body=body,
            content_type=content_type,
            match=(header_matcher({"Authorization": f"Token {test_token}"}), ),
        )
        resp = client.get("content/")
        assert resp.content == result

    def test_response_error_401(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
        test_token: str,
    ) -> None:
        url = "access401/"
        error = "Invalid token."
        mock_responses.get(
            url=f"{client._url}/{url}",
            json={"detail": error},
            status=401,
            match=(header_matcher({"Authorization": f"Token {test_token}"}), ),
        )
        with pytest.raises(InvalidTokenError, match=f"401, {error}"):
            client.get(url)

    def test_response_error_403(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
    ) -> None:
        url = "access403/"
        error = "permission error details"
        mock_responses.get(
            url=f"{client._url}/{url}",
            json={"detail": error},
            status=403,
        )
        with pytest.raises(PermissionDeniedError, match=f"403, {error}"):
            client.get(url)

    @pytest.mark.parametrize("status", (201, 204))
    def test_response(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
        status: int,
    ) -> None:
        mock_responses.get(url=f"{client._url}/access/", status=status)
        response = client.get("access/")
        assert response.status == status

    def test_response_raise_404(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
    ) -> None:
        mock_responses.get(url=f"{client._url}/access/", status=404)
        with pytest.raises(NotFoundError):
            client.get("access/")

    def test_response_raise_timeout_eror(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
    ) -> None:
        mock_responses.get(url=f"{client._url}/timeout/", body=Timeout())
        with pytest.raises(RequestTimeoutError):
            client.get("timeout/")

    def test_response_raise_status_empty(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
    ) -> None:
        mock_responses.get(url=f"{client._url}/status/", status=500)
        with pytest.raises(StatusError, match="500"):
            client.get("status/")

    def test_response_raise_status_description(
        self,
        mock_responses: 'RequestsMock',
        client: Client,
    ) -> None:
        mock_responses.get(
            url=f"{client._url}/status/",
            status=418,
            json={"error": "I'm a teapot"},
        )
        with pytest.raises(StatusError, match="418, I'm a teapot"):
            client.get("status/")



from __future__ import annotations

__all__ = ["Python"]

from typing import TYPE_CHECKING
from typing import Union
from typing import cast

from ._types import Content
from .base import BaseEndpoint

if TYPE_CHECKING:  # pragma: no cover
    from pawapi.python import Python2
    from pawapi.python import Python3


class Python(BaseEndpoint):
    """Python endpoint"""

    __slots__ = ()

    def get_python_version(self) -> Content:
        """Returns current Python version and list of available versions."""

        content = self._client.get("default_python_version/").content
        return cast(Content, content)

    def set_python_version(
        self,
        version: Union[Python2, Python3],
    ) -> Content:
        """Set default Python version.

        :param version: Python version.
        """

        result = self._client.patch(
            path="default_python_version/",
            data={"default_python_version": version.format_value(short=True)},
        )
        return cast(Content, result.content)

    def get_python3_version(self) -> Content:
        """Returns current Python3 version and list of available versions."""

        result = self._client.get("default_python3_version/")
        return cast(Content, result.content)

    def set_python3_version(self, version: Python3) -> Content:
        """Set default Python3 version.

        :param version: Python version.
        """

        result = self._client.patch(
            path="default_python3_version/",
            data={"default_python3_version": version.format_value(short=True)},
        )
        return cast(Content, result.content)

    def get_sar_version(self) -> Content:
        """Retruns current Python version used for the 'Run' button
        in the editor and list of available versions.
        """

        result = self._client.get("default_save_and_run_python_version/")
        return cast(Content, result.content)

    def set_sar_version(self, version: Python3) -> Content:
        """Set Python version used for the 'Run' button in the editor.

        :param version: Python version.
        """

        # yapf: disable
        result = self._client.patch(
            path="default_save_and_run_python_version/",
            data={"default_save_and_run_python_version": version.format_value(short=True)},  # noqa: E501
        )
        # yapf: enable
        return cast(Content, result.content)



from __future__ import annotations

__all__ = ["BaseEndpoint"]

from typing import TYPE_CHECKING

if TYPE_CHECKING:  # pragma: no cover
    from pawapi.client import Client


class BaseEndpoint:
    """Base class for endpoints.

    :param client: HTTP client.
    """

    __slots__ = "_client"

    def __init__(self, client: Client) -> None:
        self._client = client

    def __repr__(self) -> str:  # pragma: no cover
        return f"<{self.__class__.__name__}>"



from __future__ import annotations

__all__ = ["Webapp"]

from typing import TYPE_CHECKING
from typing import Dict
from typing import List
from typing import Optional
from typing import Union
from typing import cast

from pawapi.exceptions import BadRequestError
from pawapi.exceptions import StatusError

from ._types import Content
from .base import BaseEndpoint

if TYPE_CHECKING:  # pragma: no cover
    from pawapi.python import Python2
    from pawapi.python import Python3


class Webapp(BaseEndpoint):
    """Web app endpoint"""

    __path = "webapps"

    __slots__ = ()

    def list(self) -> List[Content]:
        """List all webapps."""

        result = self._client.get(f"{self.__path}/")
        return cast(List[Content], result.content)

    def create(
        self,
        domain_name: str,
        python_version: Union[Python2, Python3],
    ) -> bool:
        """Create a new webapp.

        .. note::

            Use :meth:`update` to modify configuration.

        :param domain_name: App domain name.
        :param python_version: App Python version.
        :returns: True on success.
        """

        result = self._client.post(
            path="webapps/",
            data={
                "domain_name": domain_name,
                "python_version": python_version.format_value(dot=False),
            },
        )
        return result.status == 200

    def get_info(self, domain_name: str) -> Content:
        """Returns app configuration.

        :param domain_name: App domain name.
        """

        result = self._client.get(f"{self.__path}/{domain_name}/")
        return cast(Content, result.content)

    def update(
        self,
        domain_name: str,
        *,
        python_version: Optional[Union[Python2, Python3]] = None,
        source_directory: Optional[str] = None,
        virtualenv_path: Optional[str] = None,
        force_https: Optional[bool] = None,
        protection: Optional[bool] = None,
        protection_username: Optional[str] = None,
        protection_password: Optional[str] = None,
    ) -> Content:
        """Modify app configuration.

        .. note::

            Restart the webapp is required to apply the changes.

        :param domain_name: App domain name.
        :param python_version: App Python version.
        :param source_directory: Path to app source directory.
        :param virtualenv_path: Path to app virtualenv.
        :param force_https: Redirect http to https.
        :param protection: Use password protection.
        :param protection_username: Username for password protection.
        :param protection_password: Password for password protection.
        :returns: App configuration.
        """

        pyver = None
        if python_version is not None:
            pyver = python_version.format_value(short=True)

        result = self._client.patch(
            path=f"{self.__path}/{domain_name}/",
            data={
                "python_version": pyver,
                "source_directory": source_directory,
                "virtualenv_path": virtualenv_path,
                "force_https": force_https,
                "password_protection_enabled": protection,
                "password_protection_username": protection_username,
                "password_protection_password": protection_password,
            },
        )
        return cast(Content, result.content)

    def delete(self, domain_name: str) -> bool:
        """Delete the webapp.

        .. note::

            Config is backed up in /var/www, and your code is not touched.

        :param domain_name: App domain name.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{domain_name}/")
        return result.status == 204

    def enable(self, domain_name: str) -> bool:
        """Enable the webapp.

        :param domain_name: App domain name.
        :returns: True on success.
        """

        try:
            result = self._client.post(f"{self.__path}/{domain_name}/enable/")
        # returns 400 if you try to enable an already enabled app
        except BadRequestError as error:
            if error.description is not None:
                if "cannot create any more webapps" in error.description:
                    return True
            raise  # pragma: no cover

        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"

    def disable(self, domain_name: str) -> bool:
        """Disable the webapp.

        :param domain_name: App domain name.
        :returns: True on success.
        """

        result = self._client.post(f"{self.__path}/{domain_name}/disable/")
        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"

    def reload(self, domain_name: str) -> bool:
        """Reload the webapp.

        :param domain_name: App domain name.
        :returns: True on success.
        """

        try:
            result = self._client.post(f"{self.__path}/{domain_name}/reload/")
        # returns status 500 if you try to reload disabled app
        except StatusError:
            return False

        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"

    def get_ssl_info(self, domain_name: str) -> Content:
        """Returns info about app's certificate.

        :param domain_name: App domain name.
        """

        result = self._client.get(f"{self.__path}/{domain_name}/ssl/")
        return cast(Content, result.content)

    def add_ssl(
        self,
        domain_name: str,
        cert: str,
        private_key: str,
    ) -> bool:
        """Set new certificate.

        :param domain_name: App domain name.
        :param cert: Certificate.
        :param private_key: Certificate private key.
        :returns: True on success.
        """

        result = self._client.post(
            path=f"{self.__path}/{domain_name}/ssl/",
            data={
                "cert": cert,
                "private_key": private_key,
            },
        )
        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"

    def delete_ssl(self, domain_name: str) -> bool:
        """Delete webapp's SSL.

        :param domain_name: App domain name.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{domain_name}/ssl/")
        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"

    def list_static_files(self, domain_name: str) -> List[Content]:
        """List all the static file mappings for a domain.

        :param domain_name: App domain name.
        """

        result = self._client.get(f"{self.__path}/{domain_name}/static_files/")
        return cast(List[Content], result.content)

    def add_static_file(
        self,
        domain_name: str,
        url: str,
        path: str,
    ) -> Content:
        """Create a new static files mapping.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param url: Static files URL.
        :param path: Path to static files directory.
        """

        result = self._client.post(
            path=f"{self.__path}/{domain_name}/static_files/",
            data={
                "url": url,
                "path": path,
            },
        )
        return cast(Content, result.content)

    def get_static_file_info(
        self,
        domain_name: str,
        file_id: int,
    ) -> Content:
        """Get static files info.

        URL and path of a particular mapping.

        :param domain_name: App domain name.
        :param file_id: Static files id (see `Webapp.list_static_files()`).
        """

        result = self._client.get(
            f"{self.__path}/{domain_name}/static_files/{file_id}/"
        )
        return cast(Content, result.content)

    def update_static_file(
        self,
        domain_name: str,
        file_id: int,
        url: str,
        path: str,
    ) -> Content:
        """Modify a static files mapping.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param file_id: Static files id.
        :param url: Static files URL.
        :param path: Path to static files directory.
        """

        result = self._client.patch(
            path=f"{self.__path}/{domain_name}/static_files/{file_id}/",
            data={
                "url": url,
                "path": path,
            },
        )
        return cast(Content, result.content)

    def delete_static_file(self, domain_name: str, file_id: int) -> bool:
        """Remove a static files mapping.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param file_id: Static files id.
        :returns: True on success.
        """

        result = self._client.delete(
            f"{self.__path}/{domain_name}/static_files/{file_id}/"
        )
        return result.status == 204

    def list_static_headers(self, domain_name: str) -> List[Content]:
        """List all the static headers for a domain.

        :param domain_name: App domain name.
        """

        result = self._client.get(
            f"{self.__path}/{domain_name}/static_headers/"
        )
        return cast(List[Content], result.content)

    def add_static_header(
        self,
        domain_name: str,
        url: str,
        name: str,
        value: str,
    ) -> Content:
        """Create a new static header.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param url: Header url.
        :param name: Header name.
        :param value: Header value.
        """

        result = self._client.post(
            path=f"{self.__path}/{domain_name}/static_headers/",
            data={
                "url": url,
                "name": name,
                "value": value,
            },
        )
        return cast(Content, result.content)

    def get_static_header_info(
        self,
        domain_name: str,
        header_id: int,
    ) -> Content:
        """Get header info.

        :param domain_name: App domain name.
        :param header_id: Header id.
        """

        result = self._client.get(
            f"{self.__path}/{domain_name}/static_headers/{header_id}/"
        )
        return cast(Content, result.content)

    def update_static_header(
        self,
        domain_name: str,
        header_id: int,
        url: str,
        name: str,
        value: str,
    ) -> Content:
        """Modify a static header.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param header_id: Header id.
        :param url: Header url.
        :param name: Header name.
        :param value: Header value.
        """

        result = self._client.patch(
            path=f"{self.__path}/{domain_name}/static_headers/{header_id}/",
            data={
                "url": url,
                "name": name,
                "value": value,
            },
        )
        return cast(Content, result.content)

    def delete_static_header(
        self,
        domain_name: str,
        header_id: int,
    ) -> bool:
        """Remove a static header.

        .. note::

            Webapp restart required.

        :param domain_name: App domain name.
        :param header_id: Header id.
        :returns: True on success.
        """

        result = self._client.delete(
            f"{self.__path}/{domain_name}/static_headers/{header_id}/"
        )
        return result.status == 204



from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from responses import RequestsMock


@pytest.mark.usefixtures("paw_api_client")
class TestCpu:

    def test_info(self, mock_responses: 'RequestsMock') -> None:
        mock_responses.get(f"{self.url}/cpu/", json={})
        res = self.api.cpu.get_info()
        assert isinstance(res, dict), res



from __future__ import annotations

__all__ = ["File"]

from typing import Dict
from typing import List
from typing import Optional
from typing import cast

from pawapi.exceptions import BadRequestError
from pawapi.exceptions import NotFoundError

from ._types import Content
from .base import BaseEndpoint


class File(BaseEndpoint):
    """File endpoint"""

    __path = "files"

    __slots__ = ()

    def get_file_content(self, path: str) -> Optional[bytes]:
        """Retruns file contents.

        :param path: Path to file.
        :returns: File content or None if path is not a file.
        """

        content = self._client.get(f"{self.__path}/path{path}").content
        return cast(bytes, content) if isinstance(content, bytes) else None

    def get_directory_content(self, path: str) -> Optional[Content]:
        """Returns directory content.

        :param path: Path to directory.
        :returns: Directory content or None if path is not a directory.
        """

        content = self._client.get(f"{self.__path}/path{path}").content
        return cast(Content, content) if isinstance(content, dict) else None

    def upload(self, path: str, file_content: bytes) -> bool:
        """Uploads a file.

        :param path: File destination.
        :param file_content: File content.
        :returns: True if file has been created, False if file has been updated.
        """

        result = self._client.post(
            path=f"{self.__path}/path{path}",
            files={"content": file_content},
        )
        return result.status == 201

    def delete(self, path: str) -> bool:
        """Remove file or directory.

        .. warning::

            Directory will be removed recursively.

        :param path: Path to file or directory.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/path{path}")
        return result.status == 204

    def start_sharing(self, path: str) -> str:
        """Share content.

        :param path: Path to file or directory.
        :returns: URL to shared content.
        """

        result = self._client.post(
            path=f"{self.__path}/sharing/",
            data={"path": path},
        )
        content = cast(Dict[str, str], result.content)
        return content["url"]

    def get_sharing_status(self, path: str) -> Optional[str]:
        """Sharing status for a path.

        :param path: Path to file or directory.
        :returns: URL to shared content or None.
        """

        try:
            result = self._client.get(
                f"{self.__path}/sharing/",
                {"path": path},
            )
        except NotFoundError:
            return None

        content = cast(Dict[str, str], result.content)
        return content["url"]

    def stop_sharing(self, path: str) -> bool:
        """Stop sharing.

        :param path: Path to file or directory.
        :returns: True on success.
        """

        result = self._client.delete(
            path=f"{self.__path}/sharing/",
            params={"path": path},
        )
        return result.status == 204

    def get_tree(self, path: str) -> Optional[List[str]]:
        """Returns a list of the contents of a directory, and its
        subdirectories (up to 1000 entries). Paths ending in slash ("/")
        represent directories.

        :param path: Path to directory.
        :returns: Directory content or None if path is a file or does not exist.
        """

        try:
            result = self._client.get(
                f"{self.__path}/tree/",
                {"path": path},
            )
        except BadRequestError:  # returns status 400 if path does not exist
            return None

        return cast(List[str], result.content)



from __future__ import annotations

__all__ = ["System"]

from typing import TYPE_CHECKING
from typing import cast

from ._types import Content
from .base import BaseEndpoint

if TYPE_CHECKING:  # pragma: no cover
    from pawapi.system import SystemImage


class System(BaseEndpoint):
    """System endpoint"""

    __path = "system_image"

    __slots__ = ()

    def get_current_image(self) -> Content:
        """Returns current system image and list of available images."""

        result = self._client.get(f"{self.__path}/")
        return cast(Content, result.content)

    def set_image(self, system_image: SystemImage) -> Content:
        """Set system image.

        .. note::

            Use :meth:`System.get_current_image()` to get list of available
            system images.

        :param system_image: System image name.
        """

        result = self._client.patch(
            path=f"{self.__path}/",
            data={"system_image": system_image.value},
        )
        return cast(Content, result.content)



from __future__ import annotations

from .exceptions import InvalidCredentialsError


def validate_credentials(name: str, value: str) -> None:
    if not isinstance(value, str):
        raise InvalidCredentialsError(
            f"{name} must be a str! {name} is {type(value)!r}"
        )
    if value is None or not value:
        raise InvalidCredentialsError(
            f"{name} can't be empty or None! {name} = {value!r}"
        )
    for w in value:
        if w.isspace() or (not w.isalpha() and not w.isdigit()):
            raise InvalidCredentialsError(
                f"{name} has an invalid symbol {w!r}! {name} = {value!r}"
            )



from __future__ import annotations

__all__ = [
    "ScheduledTask",
    "AlwaysOnTask",
]

from typing import List
from typing import Optional
from typing import Union
from typing import cast

from pawapi.interval import TaskInterval

from ._types import Content
from .base import BaseEndpoint


class ScheduledTask(BaseEndpoint):
    """Scheduled task endpoint"""

    __path = "schedule"

    __slots__ = ()

    def list(self) -> List[Content]:
        """List all tasks."""

        result = self._client.get(f"{self.__path}/")
        return cast(List[Content], result.content)

    def create(
        self,
        command: str,
        minute: Union[int, str],
        *,
        hour: Optional[Union[int, str]] = None,
        enabled: bool = True,
        interval: TaskInterval = TaskInterval.DAILY,
        description: Optional[str] = None,
    ) -> Content:
        """Create a new task.

        :param command: Command to execute.
        :param minute: Start time (hourly task).
        :param hour: Start time (daily task).
        :param enabled: Enable task.
        :param interval: Task start interval.
        :param description: Task description.
        :returns: Task information.
        """

        result = self._client.post(
            path=f"{self.__path}/",
            data={
                "command": command,
                "enabled": enabled,
                "interval": interval.value,
                "hour": hour,
                "minute": minute,
                "description": description,
            },
        )
        return cast(Content, result.content)

    def get_info(self, task_id: Union[int, str]) -> Content:
        """Returns information about task.

        :param task_id: Task id.
        """

        result = self._client.get(f"{self.__path}/{task_id}/")
        return cast(Content, result.content)

    def update(
        self,
        task_id: Union[int, str],
        *,
        command: Optional[str] = None,
        enabled: Optional[bool] = None,
        interval: Optional[TaskInterval] = None,
        hour: Optional[Union[int, str]] = None,
        minute: Optional[Union[int, str]] = None,
        description: Optional[str] = None,
    ) -> Content:
        """Update task.

        :param task_id: Task id.
        :param command: Command to execute.
        :param minute: Start time (hourly task).
        :param hour: Start time (daily task).
        :param enabled: Enable task.
        :param interval: Task start interval.
        :param description: Task description.
        :returns: Task information.
        """

        result = self._client.patch(
            path=f"{self.__path}/{task_id}/",
            data={
                "command": command,
                "enabled": enabled,
                "interval": interval.value if interval is not None else None,
                "hour": hour,
                "minute": minute,
                "description": description,
            },
        )
        return cast(Content, result.content)

    def delete(self, task_id: Union[int, str]) -> bool:
        """Stop and delete task.

        :param task_id: Task id.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{task_id}/")
        return result.status == 204


class AlwaysOnTask(BaseEndpoint):
    """Always-on task endpoint"""

    __path = "always_on"

    __slots__ = ()

    def list(self) -> List[Content]:
        """List all tasks."""

        result = self._client.get(f"{self.__path}/")
        return cast(List[Content], result.content)

    def create(
        self,
        command: str,
        *,
        enabled: bool = True,
        description: Optional[str] = None,
    ) -> Content:
        """Create a new task.

        :param command: Command to execute.
        :param enabled: Enable task.
        :param description: Task description.
        :returns: Task information.
        """

        result = self._client.post(
            path=f"{self.__path}/",
            data={
                "command": command,
                "description": description,
                "enabled": enabled,
            },
        )
        return cast(Content, result.content)

    def get_info(self, task_id: Union[int, str]) -> Content:
        """Returns information about task."""

        result = self._client.get(f"{self.__path}/{task_id}/")
        return cast(Content, result.content)

    def update(
        self,
        task_id: Union[int, str],
        *,
        command: Optional[str] = None,
        description: Optional[str] = None,
        enabled: Optional[bool] = None,
    ) -> Content:
        """Update task.

        :param task_id: Task id.
        :param command: Command to execute.
        :param enabled: Enable task.
        :param description: Task description.
        :returns: Task information.
        """

        result = self._client.patch(
            path=f"{self.__path}/{task_id}/",
            data={
                "command": command,
                "description": description,
                "enabled": enabled,
            },
        )
        return cast(Content, result.content)

    def delete(self, task_id: Union[int, str]) -> bool:
        """Stop and delete task.

        :param task_id: Task id.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{task_id}/")
        return result.status == 204

    def restart(
        self,
        task_id: Union[int, str],
        *,
        command: Optional[str] = None,
        description: Optional[str] = None,
        enabled: Optional[bool] = None,
    ) -> Content:
        """Restart task.

        :param task_id: Task id.
        :param command: Command to execute.
        :param enabled: Enable task.
        :param description: Task description.
        :returns: Task information
        """

        result = self._client.post(
            path=f"{self.__path}/{task_id}/restart/",
            data={
                "command": command,
                "description": description,
                "enabled": enabled,
            },
        )
        return cast(Content, result.content)



from __future__ import annotations

from typing import Optional


class PythonAnywhereAPIException(Exception):
    """Base server side exception.

    :param status_code: HTTP status code.
    :param description: Error description.
    :param raw_content: Raw response content.
    """

    def __init__(
        self,
        status_code: int,
        description: Optional[str] = None,
        raw_content: Optional[bytes] = None,
    ) -> None:
        message = str(status_code)
        if description:
            message = f"{message}, {description}"
        super().__init__(message)
        self.status_code = status_code
        self.description = description
        self.raw_content = raw_content

    def __reduce__(self):  # pragma: no cover
        return (
            self.__class__,
            (
                self.status_code,
                self.description,
                self.raw_content,
            ),
        )


class BadRequestError(PythonAnywhereAPIException):
    """Invalid request (status code 400)"""


class InvalidTokenError(PythonAnywhereAPIException):
    """Invalid API token (status code 401)"""


class PermissionDeniedError(PythonAnywhereAPIException):
    """Wrong username or somthing else (status code 403)"""


class NotFoundError(PythonAnywhereAPIException):
    """Path not found (status code 404)"""


class StatusError(PythonAnywhereAPIException):
    """Status code >= 400 (except 401, 403, 404)"""


class PawapiException(Exception):
    """Base client side exception"""


class InvalidJSONError(PawapiException):
    """Error while parsing json response"""


class InvalidCredentialsError(PawapiException):
    """Username or Token has an invalid character"""


class RequestTimeoutError(PawapiException):
    """Request timed out"""



__all__ = [
    "Console",
    "Cpu",
    "File",
    "Python",
    "Students",
    "System",
    "AlwaysOnTask",
    "ScheduledTask",
    "Webapp",
]

from .console import Console
from .cpu import Cpu
from .file import File
from .python import Python
from .students import Students
from .system import System
from .task import AlwaysOnTask
from .task import ScheduledTask
from .webapp import Webapp



__all__ = [
    "Pawapi",
    "Region",
    "Python2",
    "Python3",
    "Shell",
    "SystemImage",
    "TaskInterval",
    "__version__",
]

from logging import NullHandler
from logging import getLogger

from .interval import TaskInterval
from .pawapi import Pawapi
from .python import Python2
from .python import Python3
from .region import Region
from .shell import Shell
from .system import SystemImage
from .version import __version__

getLogger(__name__).addHandler(NullHandler())



from __future__ import annotations

__all__ = ["Console"]

from typing import TYPE_CHECKING
from typing import Dict
from typing import List
from typing import Optional
from typing import Union
from typing import cast

from ._types import Content
from .base import BaseEndpoint

if TYPE_CHECKING:  # pragma: no cover
    from pawapi.shell import Shell


class Console(BaseEndpoint):
    """Console endpoint"""

    __path = "consoles"

    __slots__ = ()

    def list(self) -> List[Content]:
        """Returns list of all your consoles."""

        content = self._client.get(f"{self.__path}/").content
        return cast(List[Content], content)

    def list_shared(self) -> List[Content]:
        """Returns list of all consoles shared with you."""

        result = self._client.get(f"{self.__path}/shared_with_you/")
        return cast(List[Content], result.content)

    def create(
        self,
        executable: Shell,
        *,
        arguments: Optional[str] = None,
        working_directory: Optional[str] = None,
    ) -> Optional[Content]:
        """Create a new console.

        .. note::

            Does not actually start the process. Open new console in the
            browser to initialize it.

        :param executable: Console executable (bash, pypy, etc.).
        :param arguments: Sturtup arguments for executable.
        :param working_directory: Startup directory for `executable`.
        :returns: Console info on success.
        """

        result = self._client.post(
            path=f"{self.__path}/",
            data={
                "executable": executable.value,
                "arguments": arguments,
                "working_directory": working_directory,
            },
        )
        return cast(Content, result.content) if result.status == 201 else None

    def get_info(self, console_id: Union[int, str]) -> Content:
        """Returns information about a console.

        :param console_id: Console id.
        """

        result = self._client.get(f"{self.__path}/{console_id}/")
        return cast(Content, result.content)

    def kill(self, console_id: Union[int, str]) -> bool:
        """Kill a console.

        :param console_id: Console id.
        :returns: True on success.
        """

        result = self._client.delete(f"{self.__path}/{console_id}/")
        return result.status == 204

    def get_output(self, console_id: Union[int, str]) -> str:
        """Returns recent command output (up to 500 characters).

        :param console_id: Console id.
        """

        result = self._client.get(
            f"{self.__path}/{console_id}/get_latest_output/"
        )
        content = cast(Dict[str, str], result.content)
        return content.get("output", "")

    def send_input(
        self,
        console_id: Union[int, str],
        command: str,
    ) -> bool:
        """Send input into the console.

        .. note::

            Add a '\\n' at the end of ``command`` to actually execute it.

        :param console_id: Console id.
        :param command: Command to execute.
        :returns: True on success.
        """

        result = self._client.post(
            path=f"{self.__path}/{console_id}/send_input/",
            data={"input": command},
        )
        content = cast(Dict[str, str], result.content)
        return content["status"] == "OK"
